﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Forms = System.Windows.Forms;
using System.IO;

namespace LegoVezZobrazovac
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        public void ZobrazPatro(object s, RoutedEventArgs e)
        {
            LevelWindow lw = new LevelWindow();
            lw.Show();
        }
        
        public void OpenFileDialog(object s, RoutedEventArgs args)
        {
            Forms.OpenFileDialog ofd = new Forms.OpenFileDialog();
            ofd.CheckFileExists = true;
            ofd.CheckPathExists = true;

            if (ofd.ShowDialog() == Forms.DialogResult.OK)
            {
                OpenFile(ofd.FileName);
            }
        }

        // all models belonging to a certain level are stored in here so they can be easily hidden on user request
        private Dictionary<int, List<ModelVisual3D>> LevelStorage = new Dictionary<int, List<ModelVisual3D>>();

        private void OpenFile(string path)
        {
            bool firstRead = false;

            using (StreamReader sr = new StreamReader(path))
            {
                while (!sr.EndOfStream)
                {
                    string line = sr.ReadLine().Trim();

                    if (line.Length == 0)
                    {
                        continue;
                    }

                    string[] data = Helper.SplitLine(line);

                    if (!firstRead) // first line - load basic params
                    {
                        TowerInfo.Width = Int32.Parse(data[0]);
                        TowerInfo.Height = Int32.Parse(data[1]);
                        TowerInfo.Bricks.Clear();
                        firstRead = true;

                        continue;
                    }

                    BrickInfo bi = new BrickInfo();
                    bi.Level = Int32.Parse(data[0]);
                    bi.X = Int32.Parse(data[1]);
                    bi.Y = Int32.Parse(data[2]);

                    switch (data[3])
                    {
                        case "A":
                            bi.Color = BrickColor.Azure;
                            break;

                        case "R":
                            bi.Color = BrickColor.Red;
                            break;

                        case "G":
                            bi.Color = BrickColor.Green;
                            break;

                        case "B":
                            bi.Color = BrickColor.Blue;
                            break;

                        case "Y":
                            bi.Color = BrickColor.Yellow;
                            break;
                    }

                    string[] rozmery = data[4].ToLower().Split('x');
                    bi.Width = Int32.Parse(rozmery[0]);
                    bi.Height = Int32.Parse(rozmery[1]);

                    TowerInfo.Bricks.Add(bi);
                }
            }

            // bricks are loaded, now add them to the viewport

            double x = -1; // used to store coordinates of the first brick (camera look direction calculation)
            double y = -1;

            while (this.viewport.Children.Count > 1)
            {
                this.viewport.Children.RemoveAt(1);
            }

            this.LevelStorage.Clear();

            foreach (BrickInfo bi in TowerInfo.Bricks)
            {
                ModelVisual3D mdl = new ModelVisual3D();

                GeometryModel3D gm = (FindResource("Kosticka") as GeometryModel3D).Clone();
                gm.Material = new DiffuseMaterial(bi.ColorBrush);
                mdl.Content = gm;

                Transform3DGroup grp = new Transform3DGroup();

                ScaleTransform3D st = new ScaleTransform3D(bi.Width, bi.Height, 1);
                TranslateTransform3D tt = new TranslateTransform3D(bi.X, bi.Y, bi.Level - 1);
                ScaleTransform3D floorHider = new ScaleTransform3D(1, 1, 1);
                RotateTransform3D rotator = new RotateTransform3D(new AxisAngleRotation3D(new Vector3D(1, 0, 0), 0));

                grp.Children.Add(st);
                grp.Children.Add(tt);
                grp.Children.Add(floorHider);
                grp.Children.Add(rotator);

                if (x == -1) // if this is the first brick added
                {
                    x = bi.X;
                    y = bi.Y;
                }

                mdl.Transform = grp;
                viewport.Children.Add(mdl);

                if (!LevelStorage.ContainsKey(bi.Level))
                {
                    LevelStorage.Add(bi.Level, new List<ModelVisual3D>());
                }

                LevelStorage[bi.Level].Add(mdl);
            }

            camera.Position = new Point3D(TowerInfo.Width + 4, TowerInfo.Height + 4, 20);
            camera.LookDirection = new Vector3D(x - camera.Position.X, y - camera.Position.Y, 0 - camera.Position.Y);  
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int choice = (int)e.AddedItems[0];

            foreach (KeyValuePair<int, List<ModelVisual3D>> levelData in this.LevelStorage)
            {
                foreach (ModelVisual3D mdl in levelData.Value)
                {
                    Transform3DGroup grp = (Transform3DGroup)mdl.Transform;

                    if (choice == -1 || levelData.Key == choice)
                    {
                        grp.Children[2] = new ScaleTransform3D(1, 1, 1);
                    }
                    else
                    {
                        grp.Children[2] = new ScaleTransform3D(0, 0, 0);
                    }
                }
            }
        }

        private double rotAngle = 0;

        private void Slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            rotAngle = e.NewValue;

            for (int i = 1; i < viewport.Children.Count; i++)
            {
                ModelVisual3D mdl = (ModelVisual3D)viewport.Children[i];
                Transform3DGroup grp = (Transform3DGroup)mdl.Transform;

                grp.Children[3] = new RotateTransform3D(new AxisAngleRotation3D(new Vector3D(0, 0, 1), rotAngle)); // rotate every viewport item by X angles as chosen by the user
            }
        }
    }
}
